"""A modification workflow for migrating an ImportedGeantIP to an GeantIP subscription."""

from orchestrator.targets import Target
from orchestrator.types import State, UUIDstr
from orchestrator.utils.errors import ProcessFailureError
from orchestrator.workflow import StepList, done, init, step, workflow
from orchestrator.workflows.steps import resync, store_process_subscription, unsync
from orchestrator.workflows.utils import wrap_modify_initial_input_form

from gso.products import ProductName
from gso.products.product_types.nren_l3_core_service import (
    ImportedNRENL3CoreService,
    NRENL3CoreService,
    NRENL3CoreServiceType,
)
from gso.services.subscriptions import get_product_id_by_name


@step("Create imported subscription")
def import_nren_l3_core_service_subscription(subscription_id: UUIDstr) -> State:
    """Take an imported subscription, and turn it into an :term:`NREN` L3 Core Service subscription."""
    old_nren_l3_core_service = ImportedNRENL3CoreService.from_subscription(subscription_id)
    if old_nren_l3_core_service.nren_l3_core_service_type == NRENL3CoreServiceType.IMPORTED_GEANT_IP:
        new_subscription_id = get_product_id_by_name(ProductName.GEANT_IP)
    elif old_nren_l3_core_service.nren_l3_core_service_type == NRENL3CoreServiceType.IMPORTED_IAS:
        new_subscription_id = get_product_id_by_name(ProductName.IAS)
    else:
        msg = f"This {old_nren_l3_core_service.nren_l3_core_service_type} is already imported, nothing to do."
        raise ProcessFailureError(message=msg, details=old_nren_l3_core_service)
    new_subscription = NRENL3CoreService.from_other_product(old_nren_l3_core_service, new_subscription_id)  # type: ignore[arg-type]

    return {"subscription": new_subscription}


@workflow("Import NREN L3 Core Service", target=Target.MODIFY, initial_input_form=wrap_modify_initial_input_form(None))
def import_nren_l3_core_service() -> StepList:
    """Modify an imported subscription into an :term:`NREN` L3 Core Service subscription to complete the import."""
    return (
        init
        >> store_process_subscription(Target.MODIFY)
        >> unsync
        >> import_nren_l3_core_service_subscription
        >> resync
        >> done
    )
